import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, scaleTime, timeFormat, extent, line, curveNatural, curveBasis } from 'd3';

const width = 800;
const height = 425;
const margin = {
  top: 10,
  right: 30,
  bottom: 60,
  left: 80,
};
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 50;

// X-axis = Date
const xValue = (d) => d.Date;
const xAxisLabel = 'Months';

// Y-axis = Opening bid price
const yValue = (d) => d.BO;
const yAxisLabel = 'Opening bid';

const xAxisTickFormat = timeFormat("%b");

const fetchCSVData = () => {
  
  const csvUrl =
  'https://gist.githubusercontent.com/17761A04F1/6dc492901e67cdd989fd05cc6c5ce1ea/raw/f75fb2b6afefa3b7ee61efd3941d6601246eb74c/data.csv';

  
  const [data, setData] = useState(null);

  useEffect(() => {
    
    // Formating each items in the data.
    const row = (d) => {
      d.Date = new Date(d.Date),
      d.BO = +d.BO
      return d;
    };
    
  	// Formating the data in the ascending order of Dates.
    csv(csvUrl, row).then((data) => {
			setData(data
							.sort(function(a, b) {
			return d3.ascending(a.Date, b.Date)
							})
			);
    });
  }, []);
  return data;
};

// Rendering the Line
const LineChart = ({
  data,
  xScale,
  yScale,
  xValue,
  yValue,
  tooltipFormat
}) => {
    
	return(
  	<g className="marks">
    <path
      fill="none"
      stroke="black"
      d={
        line()
        .x(d => xScale(xValue(d)))
        .y(d => yScale(yValue(d)))
        .curve(curveBasis)
        (data)
        }
    />
  </g>
  );
};

// Rendering the Y-axis based on the data
const YAxis = ({ yScale, innerWidth, tickOffset = 3 }) =>
  yScale.ticks().map((tickValue) => (
    <g className="tick" transform={`translate(0,${yScale(tickValue)})`}>
      <line x2={innerWidth} />
      <text
        key={tickValue}
        style={{ textAnchor: 'end' }}
        x={-tickOffset}
        dy=".32em"
        y={yScale(tickValue)}
      >
        {tickValue}
      </text>
    </g>
  ));

// Rendering the X-axis based on the data
const XAxis = ({ xScale, innerHeight, tickFormat, tickOffset = 0 }) =>
  xScale.ticks().map((tickValue) => (
    <g className="tick"
      key={tickValue}
      transform={`translate(${xScale(tickValue)},0)`}>
      <line y2={innerHeight}/>
      <text
        style={{ textAnchor: 'middle' }}
        dy=".71em"
        y={innerHeight + tickOffset}
      >
        {tickFormat(tickValue)}
      </text>
    </g>
  ));


// Renders the Line graph
const App = () => {
  
  // Fetch the CSV data from the URI
  const data = fetchCSVData();

  if (!data) {
    return <pre>Loading data...</pre>;
  }  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  // Scaling the X axis based on the data
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
  	.nice();
  
  // Scaling the Y axis based on the data
  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
  	.nice();

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${margin.left},${margin.top})`}>
        <XAxis
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={7}
        />
        <YAxis
          yScale={yScale}
          innerWidth={innerWidth}
        />
				<LineChart
          data={data}
          xScale={xScale}
          yScale={yScale}
          xValue={xValue}
          yValue={yValue}
          tooltipFormat={xAxisTickFormat}
          circleRadius={3}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
          tickOffset={7}
        >
          {xAxisLabel}
        </text>
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${innerHeight / 2}) rotate(-90)`}
        >
          {yAxisLabel}
        </text>
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
